(* ::Package:: *)

(* ::Section:: *)
(*Define seesaw parameters*)


(* ::Text:: *)
(*Rate constants:*)


kf = 2*10^6; (* fast strand displacement rate with extended toehold *)
ks = 5*10^4; (* slow strand displacement rate with universal toehold *)
krf = 26; (* fast strand dissociation rate with universal toehold *)
krs = 1.3; (* slow strand dissociation rate with extended toehold *)
kl = 10; (* strand displacement leak rate *)


(* ::Text:: *)
(*Standard concentration 1x:*)


c = 50*10^-9; 


(* ::Section:: *)
(*Define seesaw functions*)


(* ::Text:: *)
(*Translates a seesaw gate into a list of reactions:*)


seesaw[x_,l_List,r_List]:={
(* Toehold exchange reactions *)
Outer[revrxn[w[#1,x]+g[x,w[x,#2]],g[w[#1,x],x]+w[x,#2],ks,ks]&,l,r],
(* Thresholding reactions *)
Apply[rxn[w[#,x]+th[w[#,x],x],waste,kf]&,l],
Apply[rxn[w[x,#]+th[x,w[x,#]],waste,kf]&,r],
(* Universal toehold binding reactions *)
Apply[revrxn[g[w[#,x],x]+W,g[w[#,x],x,W],kf,krf]&,l],
Apply[revrxn[g[x,w[x,#]]+W,g[W,x,w[x,#]],kf,krf]&,r],
Apply[revrxn[th[w[#,x],x]+W,th[W,w[#,x],x],kf,krs]&,l],
Apply[revrxn[th[x,w[x,#]]+W,th[x,w[x,#],W],kf,krs]&,r],
Apply[revrxn[w[#,x]+G,w[#,x,G],kf,krf]&,l],
Apply[revrxn[w[#,x]+TH,w[#,x,TH],kf,krs]&,l],
(* Leak reactions *)
Outer[rxn[g[w[#1,x],x]+w[#2,x],g[w[#2,x],x]+w[#1,x],kl]&,l,l],
Outer[rxn[g[x,w[x,#1]]+w[x,#2],g[x,w[x,#2]]+w[x,#1],kl]&,r,r]
}/.List->Sequence


(* ::Text:: *)
(*Translates a reporter into a list of reactions and the initial concentration of the reporter:*)


reporter[x_,l_]:={
rxn[w[l,x]+Rep[x],Fluor[x],ks],
revrxn[Rep[x]+W, Rep[x,W],kf,krf],
revrxn[w[l,x]+G,w[l,x,G],kf,krf],
revrxn[w[l,x]+TH,w[l,x,TH],kf,krs],
conc[Rep[x],1.5*c]
}/.List->Sequence


(* ::Text:: *)
(*Translates logic OR operation into a list of seesaw gates and the concentrations of initial species:*)


seesawOR[x1_,x2_,l_List,r_List]:=
Module[{f},
{seesaw[x1,l,{x2}],
seesaw[x2,{x1},{r,f}],
conc[g[x1,w[x1,x2]],Length[l]*c],
Outer[conc[g[x2,w[x2,#]],1*c]&,r],
conc[w[x2,f],2*Length[r]*c],
conc[th[w[x1,x2],x2],1.1*0.6*c]
}]/.List->Sequence


(* ::Text:: *)
(*Translates logic AND operation into a list of seesaw gates and the concentrations of initial species:*)


seesawAND[x1_,x2_,l_List,r_List]:=
Module[{f},
{seesaw[x1,l,{x2}],
seesaw[x2,{x1},{r,f}],
conc[g[x1,w[x1,x2]],Length[l]*c],
Outer[conc[g[x2,w[x2,#]],1*c]&,r],
conc[w[x2,f],2*Length[r]*c],
conc[th[w[x1,x2],x2],1.1*(Length[l]-1+0.2)*c]
}]/.List->Sequence


(* ::Text:: *)
(*Translates inputs with fan-out into a list of seesaw gates and the concentrations of initial species:*)


inputfanout[x_,l_,r_List]:=
Module[{f},
{seesaw[x,{l},{r,f}],
Outer[conc[g[x,w[x,#]],1*c]&,r],
conc[w[x,f],2*Length[r]*c],
conc[th[w[l,x],x],1.1*0.2*c]
}]/.List->Sequence
